// ************************************************************************************************************************ //
//
//	File:			AI.cxx
//	Description:	General settings for AI (defined by Rank, Class and Difficulty)
//	Author:			Beny
//
// ************************************************************************************************************************ //


#include <inc\all\ai.h>
#include <inc\all\ai_ini.h>



//Check if the desired weapon exists for specified rank/character (subId is weaponId)
BOOL FindWeapBySubId(dword charSubId, dword rankId, dword subId, s_weapDef *weapDef)
{
	dword	i = 0;

	if ( (subId < 'A') && (subId != SUB_RND) )
	{
		while (weapDef[i].rankId != 0)
		{
			if ( (weapDef[i].rankId == rankId) && (weapDef[i].subId == charSubId) && (weapDef[i].weapId == subId) )
				return TRUE;

			i++;
		}
	}

	return FALSE;
}


// + + + ------------------------------------------------------------------------------------------------------------------
// + + +	VC	VC	VC	VC	VC	VC	VC	VC	VC	VC	VC	VC	VC	VC	VC	VC	VC	VC	VC	VC	VC	VC	VC	VC	VC	VC	VC
// + + + ------------------------------------------------------------------------------------------------------------------


//Retrieve settings for VC player body/head by Rank, Class (and Class SubId)
BOOL vcGetPlayerIniEqp(dword rankId, dword classId, dword subId, s_PlayerIniEqp *iniEqp)
{
	char	str[STR_LENGTH];
	dword	i, ofs, rnd[16];
	s_charDef	*charDef;
	s_weapDef	*weapDef;
	s_headDef	*headDef;
	BOOL		bFirst = TRUE;


	SC_Log(3, "vcGetPlayerIniEqp(%d, %d, %d, 0x%08x)", rankId, classId, subId, iniEqp);


	if ( (rankId > RNK_MAX) || (classId > CLS_MAX) || (subId > SUB_MAX) || !iniEqp )
	{
		SC_message("vcGetPlayerIniEqp(%d, %d, %d, 0x%08x) error: bad parameter", rankId, classId, subId, iniEqp);
		return FALSE;
	}


	switch(classId)
	{
	default:
		SC_message("vcGetPlayerIniEqp(%d, %d, %d, 0x%08x) warning: usupported classId! Using default.", rankId, classId, subId, iniEqp);

	case CLS_SOLDIER:
		//hat, char & weap definition
		headDef = (s_headDef*)vcSolHead;
		charDef = (s_charDef*)vcSolDef;
		weapDef = (s_weapDef*)vcSolWeapDef;
		break;

	case CLS_MACHINEGUN:
		//hat, char & weap definition
		headDef = (s_headDef*)vcMguHead;
		charDef = (s_charDef*)vcMguDef;
		weapDef = (s_weapDef*)vcMguWeapDef;
		break;
	}
	
	//head & hat
	i = ofs = 0;
	while ( (headDef[i].rankId != 0) )
	{
		if (headDef[i].rankId == rankId)
			rnd[ofs++] = i;

		i++;
	}

	if (!ofs)
	{
		SC_message("vcGetPlayerIniEqp(%d, %d, %d, 0x%08x) error: no HEAD_INI/HEAD_EQP definition matches the parameters", rankId, classId, subId, iniEqp);
		return FALSE;
	}

	i = rnd[ rand() % ofs ];

	//head ini
	sprintf(iniEqp->headIni, headDef[i].headIni);

	//head eqp
	sprintf(iniEqp->headEqp, "%s%s.ini", HEAD_EQP_ROOT, headDef[i].hatEqp);


labelTryDefault:		//goto here if can't set specified weapon

	//body ini
	i = ofs = 0;
	while ( (charDef[i].rankId != 0) )
	{
		if ( (charDef[i].rankId == rankId) && ((charDef[i].subId == subId) || (subId == SUB_RND) || FindWeapBySubId(charDef[i].subId, rankId, subId, weapDef)) )
			rnd[ofs++] = i;

		i++;
	}

	if (!ofs)
	{
		if (bFirst)
		{
			bFirst = FALSE;
			SC_Log(2, "vcGetPlayerIniEqp(%d, %d, %d, 0x%08x) warning: specified weapon not found for this subId - returning default", rankId, classId, subId, iniEqp);

			subId = SUB_RND;
			goto labelTryDefault;
		}
		else
		{
			SC_message("vcGetPlayerIniEqp(%d, %d, %d, 0x%08x) error: no BODY definition matches the parameters", rankId, classId, subId, iniEqp);
			return FALSE;
		}
	}

	i = rnd[ rand() % ofs ];

	//
	sprintf(iniEqp->bodyIni, "%s%s", VC_INI_BASE, charDef[i].iniName);


	//body eqp
	switch(classId)
	{
	default:
	case CLS_SOLDIER:
		sprintf(str, SOL_CLS_BASE);
		break;

	case CLS_MACHINEGUN:
		sprintf(str, MGU_CLS_BASE);
		break;
	}

	sprintf(iniEqp->bodyEqp, "%s%s\\%s%d_%s%c\\eqp.ini", BODY_EQP_ROOT, iniEqp->bodyIni, VC_EQP_BASE, rankId, str, charDef[i].subId);	//subId


	//weapon
	if ( (subId) && (subId < 'A') )
	{
		iniEqp->weapon = subId;
	}
	else
	{
		i = ofs = 0;
		while ( (weapDef[i].rankId != 0) )
		{
			if ((weapDef[i].rankId == rankId))
				rnd[ofs++] = i;

			i++;
		}

		if (!ofs)
		{
			SC_message("vcGetPlayerIniEqp(%d, %d, %d, 0x%08x) error: no WEAPON definition matches the parameters", rankId, classId, subId, iniEqp);
			return FALSE;
		}

		i = rnd[ rand() % ofs ];

		//
		iniEqp->weapon = weapDef[i].weapId;
	}


	return TRUE;
}





// + + + ------------------------------------------------------------------------------------------------------------------
// + + +	US	US	US	US	US	US	US	US	US	US	US	US	US	US	US	US	US	US	US	US	US	US	US	US	US	US	US
// + + + ------------------------------------------------------------------------------------------------------------------


//Retrieve settings for US player body/head by Rank, Class (and Class SubId)
BOOL usGetPlayerIniEqp(dword rankId, dword classId, dword subId, s_PlayerIniEqp *iniEqp)
{
	char	str[STR_LENGTH];
	dword	i, ofs, rnd[16];
	s_charDef	*charDef;
	s_weapDef	*weapDef;
	s_headDef	*headDef;
	BOOL		bFirst = TRUE;


	SC_Log(3, "usGetPlayerIniEqp(%d, %d, %d, 0x%08x)", rankId, classId, subId, iniEqp);


	if ( (rankId > RNK_MAX) || (classId > CLS_MAX) || (subId > SUB_MAX) || !iniEqp )
	{
		SC_message("usGetPlayerIniEqp(%d, %d, %d, 0x%08x) error: bad parameter", rankId, classId, subId, iniEqp);
		return FALSE;
	}


	switch(classId)
	{
	default:
		SC_message("usGetPlayerIniEqp(%d, %d, %d, 0x%08x) warning: usupported classId! Using default.", rankId, classId, subId, iniEqp);

	case CLS_SOLDIER:
		//hat, char & weap definition
		headDef = (s_headDef*)usSolHead;
		charDef = (s_charDef*)usSolDef;
		weapDef = (s_weapDef*)usSolWeapDef;
		break;

	case CLS_MACHINEGUN:
		//hat, char & weap definition
		headDef = (s_headDef*)usMguHead;
		charDef = (s_charDef*)usMguDef;
		weapDef = (s_weapDef*)usMguWeapDef;
		break;
	}
	
	//head & hat
	i = ofs = 0;
	while ( (headDef[i].rankId != 0) )
	{
		if (headDef[i].rankId == rankId)
			rnd[ofs++] = i;

		i++;
	}

	if (!ofs)
	{
		SC_message("usGetPlayerIniEqp(%d, %d, %d, 0x%08x) error: no HEAD_EQP definition matches the parameters", rankId, classId, subId, iniEqp);
		return FALSE;
	}

	i = rnd[ rand() % ofs ];

	//head ini
	sprintf(iniEqp->headIni, headDef[i].headIni);

	//head eqp
	sprintf(iniEqp->headEqp, "%s%s.ini", HEAD_EQP_ROOT, headDef[i].hatEqp);


labelTryDefault:		//goto here if can't set specified weapon

	//body ini
	i = ofs = 0;
	while ( (charDef[i].rankId != 0) )
	{
		if ( (charDef[i].rankId == rankId) && ((charDef[i].subId == subId) || (subId == SUB_RND) || FindWeapBySubId(charDef[i].subId, rankId, subId, weapDef)) )
			rnd[ofs++] = i;

		i++;
	}

	if (!ofs)
	{
		if (bFirst)
		{
			bFirst = FALSE;
			SC_Log(2, "usGetPlayerIniEqp(%d, %d, %d, 0x%08x) warning: specified weapon not found for this subId - returning default", rankId, classId, subId, iniEqp);

			subId = SUB_RND;
			goto labelTryDefault;
		}
		else
		{
			SC_message("usGetPlayerIniEqp(%d, %d, %d, 0x%08x) error: no BODY definition matches the parameters", rankId, classId, subId, iniEqp);
			return FALSE;
		}
	}

	i = rnd[ rand() % ofs ];

	//
	sprintf(iniEqp->bodyIni, "%s%s", US_INI_BASE, charDef[i].iniName);


	//body eqp
	switch(classId)
	{
	default:
	case CLS_SOLDIER:
		sprintf(str, SOL_CLS_BASE);
		break;

	case CLS_MACHINEGUN:
		sprintf(str, MGU_CLS_BASE);
		break;
	}

	sprintf(iniEqp->bodyEqp, "%s%s\\%s%d_%s%c\\eqp.ini", BODY_EQP_ROOT, iniEqp->bodyIni, US_EQP_BASE, rankId, str, charDef[i].subId);	//subId


	//weapon
	if ( (subId) && (subId < 'A') )
	{
		iniEqp->weapon = subId;
	}
	else
	{
		i = ofs = 0;
		while ( (weapDef[i].rankId != 0) )
		{
			if ((weapDef[i].rankId == rankId))
				rnd[ofs++] = i;

			i++;
		}

		if (!ofs)
		{
			SC_message("usGetPlayerIniEqp(%d, %d, %d, 0x%08x) error: no WEAPON definition matches the parameters", rankId, classId, subId, iniEqp);
			return FALSE;
		}

		i = rnd[ rand() % ofs ];

		//
		iniEqp->weapon = weapDef[i].weapId;
	}


	return TRUE;
}








// + + + ------------------------------------------------------------------------------------------------------------------
// + + +	BOTH	BOTH	BOTH	BOTH	BOTH	BOTH	BOTH	BOTH	BOTH	BOTH	BOTH	BOTH	BOTH	BOTH
// + + + ------------------------------------------------------------------------------------------------------------------


//Set AI properties
BOOL SetRankClass(sc_player pl, dword difficulty, dword rankId, dword classId, dword flags)
{
	float				multDif;
	s_SC_P_AI_props		aip;
	s_SC_P_ScriptProps	sp;


	SC_Log(3, "SetRankClass(0x%08x, %d, %d, %d, 0x%08x)", pl, difficulty, rankId, classId, flags);


	if (!pl)
		return FALSE;


	//difficulty multiplier
	switch(difficulty)
	{
		case DIF_EASY:		multDif = DIF_MULT_EASY; break;
		case DIF_NORMAL:	multDif = DIF_MULT_NORMAL; break;
		case DIF_HARD:		multDif = DIF_MULT_HARD; break;
		case DIF_VIETNAM:	multDif = DIF_MULT_VIETNAM; break;

		default:
			multDif = 1.0f;
			SC_message("SetRankClass(): undefined difficulty (%d)", difficulty);
			break;
	}


	//get current ai props
	CLEAR(aip);
	SC_P_Ai_GetProps(pl, &aip);


	//HP, damage
	SC_P_GetScriptProps(pl, &sp);

	//caused damage (affected by difficulty)
	sp.caused_damage_mult	= multDif;


	//set Common AI properties (only non-default setting is here)
	if ( !(flags & PL_AI_DONTSETCOMMON) )
	{																				//Engine defaults:
		aip.vis[0].distance					= 3.0f;									//2.0
		aip.vis[0].angle_perception_lost	= 3.0f;									//3.0

		aip.vis[1].angle					= 3.14f;								//2.967 (170)
		aip.vis[1].angle_perception_lost	= 3.0f;									//3.0

		aip.vis[2].angle					= 1.75f;	//100 deg					//1.396 (80)
		aip.vis[2].angle_perception_lost	= 2.7f;									//2.5
		aip.vis[2].distance_perception_lost	= 0.8f;									//0.3

		aip.enemydead_forget_speed_mult		= 3.0f;									//2.0
		aip.danger_lookat_sure_time_min		= 1.0f;									//1.5
		aip.battlemode_tohideout_z_dist_max	= 2.0f;									//0			//added 2.9.2005
		aip.fakedangermode_tohideout_z_dist_max		= 1.0f;							//0			//-||-
		aip.enemydead_recognize_immediatelly_dist	= 7.0f;							//10.0		//-||-
		aip.allydeath_reaction_vis_dist				= 12.0f;						//10.0		//-||-
	}

	//flags
	aip.flags  = 0;																	//0
	aip.flags |= (flags & PL_AI_BLIND)		? SC_PLAI_PROPS_FL_SITUPDATE_VISUAL_DISABLED : 0;
	aip.flags |= (flags & PL_AI_DEAF)		? SC_PLAI_PROPS_FL_SITUPDATE_HEAR_DISABLED : 0;
	aip.flags |= (flags & PL_AI_NOSEND)		? SC_PLAI_PROPS_FL_SITSHARE_SEND_DISABLED : 0;
	aip.flags |= (flags & PL_AI_NORECEIVE)	? SC_PLAI_PROPS_FL_SITSHARE_RECEIVE_DISABLED : 0;
	aip.flags |= (flags & PL_AI_NOSHOOT)	? SC_PLAI_PROPS_FL_SHOOTING_DISABLED : 0;
	aip.flags |= (flags & PL_AI_HITALLY)	? SC_PLAI_PROPS_FL_SHOOTING_HIT_ALLIES : 0;
	aip.flags |= (flags & PL_AI_INVISIBLE)	? SC_PLAI_PROPS_FL_ISINVISIBLE_FOR_ENEMY_AI : 0;

	//other 'flagged' values
	aip.mwp_search_dist = (flags & PL_AI_DONTUSEMWP) ? 0 : 3.0f;					//0
	if (flags & PL_AI_DONTCHECKCORPSE)
		aip.deadbodycheck_searchdistance = 0;


	//set Rank properties
	switch(rankId)
	{
	case RNK_1:
		aip.aimtime_max					= RNK_1_AIMTIME;							//0.7
		aip.aimtime_canshoot			= RNK_1_CANSHOOT;							//0.1
		aip.aimtime_enemy_min_dist		= RNK_1_AIMTIMEDIST;						//8.0
		aip.firstweakshoot_time			= RNK_1_WEAKSHOT_TIME;						//1.5
		aip.firstweakshoot_imprec_mult	= RNK_1_WEAKSHOT_IMPR;						//2.0
		aip.danger_around_stop_shooting	= RNK_1_DNG_STOPSHOOT;						//25.0
		aip.danger_around_no_peepout	= RNK_1_DNG_DONTPEEP;						//5.0
		aip.forget_time_enemy			= RNK_1_FORGET_ENEMY;						//20.0

		//received damage
		sp.received_damage_mult			= RNK_1_RECEIVED_DMG;
		break;

	case RNK_2:
		aip.aimtime_max					= RNK_2_AIMTIME;							//0.7
		aip.aimtime_canshoot			= RNK_2_CANSHOOT;							//0.1
		aip.aimtime_enemy_min_dist		= RNK_2_AIMTIMEDIST;						//8.0
		aip.firstweakshoot_time			= RNK_2_WEAKSHOT_TIME;						//1.5
		aip.firstweakshoot_imprec_mult	= RNK_2_WEAKSHOT_IMPR;						//2.0
		aip.danger_around_stop_shooting	= RNK_2_DNG_STOPSHOOT;						//25.0
		aip.danger_around_no_peepout	= RNK_2_DNG_DONTPEEP;						//5.0
		aip.forget_time_enemy			= RNK_2_FORGET_ENEMY;						//20.0

		//received damage
		sp.received_damage_mult			= RNK_2_RECEIVED_DMG;
		break;

	case RNK_3:
		aip.aimtime_max					= RNK_3_AIMTIME;							//0.7
		aip.aimtime_canshoot			= RNK_3_CANSHOOT;							//0.1
		aip.aimtime_enemy_min_dist		= RNK_3_AIMTIMEDIST;						//8.0
		aip.firstweakshoot_time			= RNK_3_WEAKSHOT_TIME;						//1.5
		aip.firstweakshoot_imprec_mult	= RNK_3_WEAKSHOT_IMPR;						//2.0
		aip.danger_around_stop_shooting	= RNK_3_DNG_STOPSHOOT;						//25.0
		aip.danger_around_no_peepout	= RNK_3_DNG_DONTPEEP;						//5.0
		aip.forget_time_enemy			= RNK_3_FORGET_ENEMY;						//20.0

		//received damage
		sp.received_damage_mult			= RNK_3_RECEIVED_DMG;
		break;

	case RNK_4:
		aip.aimtime_max					= RNK_4_AIMTIME;							//0.7
		aip.aimtime_canshoot			= RNK_4_CANSHOOT;							//0.1
		aip.aimtime_enemy_min_dist		= RNK_4_AIMTIMEDIST;						//8.0
		aip.firstweakshoot_time			= RNK_4_WEAKSHOT_TIME;						//1.5
		aip.firstweakshoot_imprec_mult	= RNK_4_WEAKSHOT_IMPR;						//2.0
		aip.danger_around_stop_shooting	= RNK_4_DNG_STOPSHOOT;						//25.0
		aip.danger_around_no_peepout	= RNK_4_DNG_DONTPEEP;						//5.0
		aip.forget_time_enemy			= RNK_4_FORGET_ENEMY;						//20.0

		//received damage
		sp.received_damage_mult			= RNK_4_RECEIVED_DMG;
		break;

	case RNK_5:
		aip.aimtime_max					= RNK_5_AIMTIME;							//0.7
		aip.aimtime_canshoot			= RNK_5_CANSHOOT;							//0.1
		aip.aimtime_enemy_min_dist		= RNK_5_AIMTIMEDIST;						//8.0
		aip.firstweakshoot_time			= RNK_5_WEAKSHOT_TIME;						//1.5
		aip.firstweakshoot_imprec_mult	= RNK_5_WEAKSHOT_IMPR;						//2.0
		aip.danger_around_stop_shooting	= RNK_5_DNG_STOPSHOOT;						//25.0
		aip.danger_around_no_peepout	= RNK_5_DNG_DONTPEEP;						//5.0
		aip.forget_time_enemy			= RNK_5_FORGET_ENEMY;						//20.0

		//received damage
		sp.received_damage_mult			= RNK_5_RECEIVED_DMG;
		break;

	case RNK_6:
		aip.aimtime_max					= RNK_6_AIMTIME;							//0.7
		aip.aimtime_canshoot			= RNK_6_CANSHOOT;							//0.1
		aip.aimtime_enemy_min_dist		= RNK_6_AIMTIMEDIST;						//8.0
		aip.firstweakshoot_time			= RNK_6_WEAKSHOT_TIME;						//1.5
		aip.firstweakshoot_imprec_mult	= RNK_6_WEAKSHOT_IMPR;						//2.0
		aip.danger_around_stop_shooting	= RNK_6_DNG_STOPSHOOT;						//25.0
		aip.danger_around_no_peepout	= RNK_6_DNG_DONTPEEP;						//5.0
		aip.forget_time_enemy			= RNK_6_FORGET_ENEMY;						//20.0

		//received damage
		sp.received_damage_mult			= RNK_6_RECEIVED_DMG;
		break;

	default:
		SC_message("SetRankClass(): undefined rankId (%d)", rankId);
		break;
	}


	//set Class properties
	switch(classId)
	{
	case CLS_SOLDIER:
		aip.shoot_imprecision_xy					= CLS_SOLDIER_SI_XY;			//0.3
		aip.shoot_imprecision_z						= CLS_SOLDIER_SI_Z;				//0.1
		aip.shoot_imprecision_correction_time_min	= CLS_SOLDIER_SI_CORTIME_MIN;	//
		aip.shoot_imprecision_correction_time_max	= CLS_SOLDIER_SI_CORTIME_MAX;	//
		break;

	case CLS_MACHINEGUN:
		aip.shoot_imprecision_xy					= CLS_MACHINE_SI_XY;			//0.3
		aip.shoot_imprecision_z						= CLS_MACHINE_SI_Z;				//0.1
		aip.shoot_imprecision_correction_time_min	= CLS_MACHINE_SI_CORTIME_MIN;	//
		aip.shoot_imprecision_correction_time_max	= CLS_MACHINE_SI_CORTIME_MAX;	//
		break;

	case CLS_SAPPER:
		aip.shoot_imprecision_xy					= CLS_SAPPER_SI_XY;			//0.3
		aip.shoot_imprecision_z						= CLS_SAPPER_SI_Z;				//0.1
		aip.shoot_imprecision_correction_time_min	= CLS_SAPPER_SI_CORTIME_MIN;	//
		aip.shoot_imprecision_correction_time_max	= CLS_SAPPER_SI_CORTIME_MAX;	//
		break;

	case CLS_ROCKETEER:
		aip.shoot_imprecision_xy					= CLS_ROCKET_SI_XY;			//0.3
		aip.shoot_imprecision_z						= CLS_ROCKET_SI_Z;				//0.1
		aip.shoot_imprecision_correction_time_min	= CLS_ROCKET_SI_CORTIME_MIN;	//
		aip.shoot_imprecision_correction_time_max	= CLS_ROCKET_SI_CORTIME_MAX;	//
		break;

	case CLS_COMMANDER:
		aip.shoot_imprecision_xy					= CLS_COMMAND_SI_XY;			//0.3
		aip.shoot_imprecision_z						= CLS_COMMAND_SI_Z;				//0.1
		aip.shoot_imprecision_correction_time_min	= CLS_COMMAND_SI_CORTIME_MIN;	//
		aip.shoot_imprecision_correction_time_max	= CLS_COMMAND_SI_CORTIME_MAX;	//
		break;

	case CLS_SNIPER:
		aip.shoot_imprecision_xy					= CLS_SNIPER_SI_XY;			//0.3
		aip.shoot_imprecision_z						= CLS_SNIPER_SI_Z;				//0.1
		aip.shoot_imprecision_correction_time_min	= CLS_SNIPER_SI_CORTIME_MIN;	//
		aip.shoot_imprecision_correction_time_max	= CLS_SNIPER_SI_CORTIME_MAX;	//
		break;

	default:
		SC_message("SetRankClass(): undefined classId (%d)", classId);
		break;
	}


	//apply difficulty to FirstWeakShot Time setting
	aip.firstweakshoot_time	/= multDif;


	//set new ai props
	SC_P_Ai_SetProps(pl, &aip);

	//set new caused/received damage
	SC_P_SetScriptProps(pl, &sp);


	return TRUE;
}

//
BOOL GetDamage(dword difficulty, dword rankId, dword classId, s_SC_P_ScriptProps *sp)
{
	if ( (rankId > RNK_MAX) || (classId > CLS_MAX) || (!sp) )
	{
		SC_message("GetDamage(%d, %d, %d, 0x%08x) error: bad parameter", difficulty, rankId, classId, sp);
		return FALSE;
	}

	//difficulty multiplier
	switch(difficulty)
	{
		case DIF_EASY:		sp->caused_damage_mult = DIF_MULT_EASY; break;
		case DIF_NORMAL:	sp->caused_damage_mult = DIF_MULT_NORMAL; break;
		case DIF_HARD:		sp->caused_damage_mult = DIF_MULT_HARD; break;
		case DIF_VIETNAM:	sp->caused_damage_mult = DIF_MULT_VIETNAM; break;

		default:
			SC_message("GetDamage(): undefined difficulty (%d)", difficulty);
			return FALSE;
	}

	//set Rank properties
	switch(rankId)
	{
		case RNK_1:	sp->received_damage_mult = RNK_1_RECEIVED_DMG; break;
		case RNK_2:	sp->received_damage_mult = RNK_2_RECEIVED_DMG; break;
		case RNK_3:	sp->received_damage_mult = RNK_3_RECEIVED_DMG; break;
		case RNK_4:	sp->received_damage_mult = RNK_4_RECEIVED_DMG; break;
		case RNK_5:	sp->received_damage_mult = RNK_5_RECEIVED_DMG; break;
		case RNK_6:	sp->received_damage_mult = RNK_6_RECEIVED_DMG; break;

		default:
			SC_message("GetDamage(): undefined rank (%d)", rankId);
			return FALSE;
	}


	return TRUE;
}

