// ************************************************************************************************************************ //
//																															//
//	File:			Player.h																								//
//	Description:	Common header used by player scripts	    													       	//
//	Author:			Beny																									//
//																															//
// ************************************************************************************************************************ //


#ifndef	__PLAYER_H__
#define	__PLAYER_H__


#include <inc\sc_global.h>

// +++ Defines +++
// +++ Defines +++
// +++ Defines +++


//s_CreatePlayerInfo flags
#define	PL_CR_PC			0x00000001			//if set, PC-controlled player is created, otherwise AI is created
#define	PL_CR_BLOCKER		0x00000002			//set whether to create path-blocker under the character
#define	PL_CR_RESPAWN		0x00000004			//respawn immediately
#define	PL_CR_COMA			0x00000008			//enable 'coma state' (usually for allies)
#define	PL_CR_CANTDIE		0x00000010			//player will stay in comma, he won't die
#define	PL_CR_FORCE_IK_DTH	0x00000020			//player will die with IK anytime
#define	PL_CR_NODROP		0x00000040			//don't drop any items when the player is killed
#define	PL_CR_MEDIC			0x00001000			//create player as Medic (will heal others)
#define	PL_CR_ENGINEER		0x00002000			//create player as Engineer (will support others with ammo)
#define	PL_CR_RADIO			0x00004000			//create player as Radio Operator


//s_aiProps flags
#define	PL_AI_BLIND			0x00000001			//AI will not react on visual input
#define	PL_AI_DEAF			0x00000002			//-||- sonic input
#define	PL_AI_NOSEND		0x00000004			//don't send info about enemies to other group members
#define	PL_AI_NORECEIVE		0x00000008			//don't receive info about enemies from other group members
#define	PL_AI_NOSHOOT		0x00000010			//disable shooting
#define	PL_AI_HITALLY		0x00000020			//AI will hit allies
#define	PL_AI_INVISIBLE		0x00000040			//player is invisible for enemies
#define	PL_AI_DEFAULT		0x10000000			//set flags to default (0)
#define	PL_AI_NOREACTION	(PL_AI_BLIND | PL_AI_DEAF)
#define	PL_AI_SOLITAIRE		(PL_AI_NOSEND | PL_AI_NORECEIVE)


//PendlPath name base
#define	PND_NAME_BASE		"pth_%d_%d"
//TacticMove name base
#define	MOV_NAME_BASE		"mov_%d_%d"

//Enemy reaction of pendlers
#define	PND_ENEMY_REACTION_NONE		0			//do nothing
#define	PND_ENEMY_REACTION_HOLD		1			//hold current position
#define	PND_ENEMY_REACTION_ATTACK	2			//attack player



//Default settings

//member id
#ifndef	ID_MEMBER
#define	ID_MEMBER	SC_P_CREATE_MEMID_AUTO
#endif






// +++ Macros +++
// +++ Macros +++
// +++ Macros +++

//#define	SetPendl() { gMainTask = MT_PENDL;  gPhase = GP_PENDL_SET; }




// +++ Structs +++
// +++ Structs +++
// +++ Structs +++


//it's passed to CreatePlayer()
typedef struct {
	dword	flags;						//PL_CR_* flags
	dword	idSide;
	dword	idGroup;
	dword	idMember;
	dword	nameNr;
	char	*nameAnsi;
	char	*iniFile;
	char	*iniEqp;
	char	*head;
	char	*headEqp;
	char	*genVoice;
	dword	numWeap;
	s_SC_P_CreateWeap	*weap;
} s_CreatePlayerInfo;

//frequently used Ai props (for SetAiProps())
typedef struct {
	dword	flags;				//PL_AI_*
	float	visDist;			//how far can see
	float	timeEnemyForget;	//sklerosis
	float	hearDist;			//how far can hear
	float	shootImprecisionXY;	//how inaccurate he is
	float	shootImprecisionZ;	//how inaccurate he is in Z-axis
} s_aiProps;

//used by MT_PendlStart() and MT_PendlContinue()
typedef struct {
	float	moveStep;
	float	tmStartDelay;
	float	tmWait;
	float	tmWaitRnd;
} s_mtPendl;

//MT_Attack()
typedef struct {
	c_Vector3	pos;
	float		moveStep;
	float		tmStartDelay;
	float		tmWait;
	float		tmWaitRnd;
} s_mtAttack;

//MT_Move()
typedef struct {
	c_BoxOriented targetBox;
	dword		coopId;
	float		movePerc;
	float		moveStep;
	float		tmStartDelay;
	float		tmWait;
	float		tmWaitRnd;
} s_mtMove;

//used for dialogues
typedef struct {
	dword	speechId;			//id of speech
	BOOL	bPC;				//if TRUE it will be said by PC player, otherwise by THIS player
	dword	animId;				//this is number of the animation file (e.g. if animId=203, then "G\Anims\Script Anims\an203.stg" will be used)
} s_speechProp;



#endif	//__PLAYER_H__
