#define	LM_MULTIPLY_VAL	8

// transformations
float4x4 World      : WORLD;
float4x4 View       : VIEW;
float4x4 Projection : PROJECTION;
float3   dl_dir;
float3   dl_dif;
float3   dl_amb;
float	 g_alpha;
float	fog_idelta;
float	fog_end;
float	specular;
float	glossiness;

float4  texmove2;
float3	camera_loc;
float3	camera_dir;

float4	omni0_dif,omni0_loc,omni0_var;
float4	omni1_dif,omni1_loc,omni1_var;
float4	omni2_dif,omni2_loc,omni2_var;
float4	omni3_dif,omni3_loc,omni3_var;
float4	omni4_dif,omni4_loc,omni4_var;
float4	omni5_dif,omni5_loc,omni5_var;
float4	omni6_dif,omni6_loc,omni6_var;

struct s_waterinf {
	float	alphaAngle;
	float	envBlend;
	float	sharpness_dif1;
	float	sharpness_dif2;
	float	sharpness_env;
};

s_waterinf waterinf;

sampler smp0 : register(s0);
sampler smp1 : register(s1);
samplerCUBE smp_env : register(s2);
sampler smplm: register(s3);
samplerCUBE smp_nor : register(s4);

#define	VS_STDOUT	float4 Pos    : POSITION;\
					float2 Tex0   : TEXCOORD0;\
					float2 Tex1   : TEXCOORD1;\
					float2 Tex2   : TEXCOORD2;\
					float  Fog    : FOG;\
					float3 tex_XYZ: TEXCOORD6;\
					float3 Normal : TEXCOORD7

#define	PS_STDIN	float2 Tex0  : TEXCOORD0,float2 Tex1  : TEXCOORD1,float2 Tex2  : TEXCOORD2,float3 pos_XYZ : TEXCOORD6, float3 Normal:TEXCOORD7

struct STD_OUT
{
	float4 Pos;
	float4 WorldPos;
	float2 Tex0;
	float2 Tex1;
	float2 Tex2;
	float3 Normal;
	float  Fog;
};

#define	VS_INPUT	float3 Pos  : POSITION, float3 Norm : NORMAL, float2 Tex0  : TEXCOORD0, float2 Tex1  : TEXCOORD1, float2 Tex2  : TEXCOORD2

#define	STDOUT		std_out = ComputeStdOut(Pos, Norm, Tex0, Tex1, Tex2);\
					Out.Pos = std_out.Pos;\
					Out.Normal = std_out.Normal;\
					Out.Tex0 = std_out.Tex0;\
					Out.Tex1 = std_out.Tex1;\
					Out.Tex2 = std_out.Tex2;\
					Out.Fog = std_out.Fog


STD_OUT ComputeStdOut(float3 pos, float3 norm, float2 tex0, float2 tex1, float2 tex2)
{
	STD_OUT Out;
	Out.Pos = float4(pos,1);

	float f;
	Out.WorldPos = mul(float4(pos, 1), World);
	Out.Normal = normalize(mul(norm, (float3x3)World));
	
	float3 P = mul(Out.WorldPos, View);  // position (view space)

	f = (fog_end - P.z) * fog_idelta;
	Out.Fog = clamp(f,0,1);
	Out.Pos  = mul(float4(P, 1), Projection);             // position (projected)
	Out.Tex0  = tex0;
	Out.Tex1  = tex1;
	Out.Tex2  = tex2;

	return Out;
}

float3 DirLight(float3 n)
{
	float cosalpha = dot(n, -dl_dir);
	if (cosalpha < 0) cosalpha = 0;
	return dl_amb + dl_dif * cosalpha;
}

float3 OmniDistance(float4 wpos, float4 oloc, float orad)
{
	return (wpos-oloc) / orad;
}

float3 CubeNormalize(float3 vec)
{
	return 2.0 * texCUBE(smp_nor, vec) - 1.0;
}

float3 OmniLight(float3 n, float4 dif, float4 dist, float range, float c_att, float l_att)
{
	float4 w;
	float q,x,y;
	float clamped;
	float3 idist;
	w = dist*dist;
	q = 1-(w.x+w.y+w.z);
	if (q >= 0) x = c_att; else x = 0;
	y = q * l_att + x;
	dif *= y;
	clamped = saturate(dot(CubeNormalize(dist.xyz),-n));
	w = saturate(dif * clamped); //SAT
	return w.xyz;
}

float3 c_dir;

float3 Specular(float3 n0, float3 n, float3 xyz, float spec_tex)
{
	c_dir = CubeNormalize(xyz - camera_loc);
	
	float3 r_vec; // reflection vector

	r_vec = reflect(dl_dir, n);
	float specmul = saturate(dot(-c_dir, r_vec));
	float cosalpha = pow(specmul, glossiness) * specular * spec_tex;
	return float3(cosalpha,cosalpha,cosalpha) * dl_dif;
}

/////////////////////////////////////////
//                                     //
// MAIN SHADER PROGRAM - 0 Omni lights //
//                                     //
/////////////////////////////////////////
struct VS_OUTPUT_0L
{
	VS_STDOUT;
};

VS_OUTPUT_0L VS(VS_INPUT)
{
	VS_OUTPUT_0L Out;
	STD_OUT std_out;

	STDOUT;
	
	Out.tex_XYZ = std_out.WorldPos;
	
	return Out;
}

float4 PS(
    PS_STDIN
    ) : COLOR
{
	float3 n;
	float2 add_n;
	float4 texcol,tex_n;

	n = Normal;

	float2 t_ld = Tex1;
	t_ld.x += texmove2.x;
	t_ld.y += texmove2.y;
	tex_n = tex2D(smp1, t_ld);

	add_n.x = (tex_n.r - 0.5f)/2;
	add_n.y = (tex_n.g - 0.5f)/2;

	float3 n0 = CubeNormalize(Normal);

	n = CubeNormalize(n);

	float3 spec = Specular(n0, n, pos_XYZ, tex_n.a);

	texcol = tex2D(smp0, Tex0);

	float3 env_z = reflect(c_dir, n0) + (tex_n.r - tex_n.g) / 2;
	float3 env_xyz = reflect(c_dir, n);
	env_xyz.z = (env_z.z * 2 - 1) * 0.9 + 0.1;

	float4 lm_val = tex2D(smplm, Tex2);lm_val = lm_val * lm_val.a * LM_MULTIPLY_VAL;
	lm_val.a=g_alpha;

	float4 retv = texcol*waterinf.envBlend +texCUBE(smp_env, env_xyz)*(1-waterinf.envBlend);
	retv.xyz += spec;

	retv.a = g_alpha;

//	float4 retv;
//	retv.xyz = texcol.www;
//	retv.w = 1;
	return retv * lm_val;
}

technique tech_0L
{
	pass P0 {
		AlphaBlendEnable = TRUE;
		AlphaTestEnable = FALSE;
		VertexShader = compile vs_1_1 VS();
		PixelShader = compile ps_2_0 PS();
	}
}
