// transformations
float4x4 World      : WORLD;
float4x4 View       : VIEW;
float4x4 Projection : PROJECTION;
float3   dl_dir;
float3   dl_dif;
float3   dl_amb;
float	 g_alpha;
float3	camera_loc;
float	specular;
float	glossiness;
float	fog_idelta;
float	fog_end;
float4	omni0_dif,omni0_loc,omni0_var;
float4	omni1_dif,omni1_loc,omni1_var;
float4	omni2_dif,omni2_loc,omni2_var;
float4	omni3_dif,omni3_loc,omni3_var;
float4	omni4_dif,omni4_loc,omni4_var;
float4	omni5_dif,omni5_loc,omni5_var;
float4	omni6_dif,omni6_loc,omni6_var;

sampler smp0 : register(s0);
sampler smp1_nm : register(s1);
sampler smp_nor : register(s2);

#define	VS_STDOUT	float4 Pos     : POSITION;\
					float2 Tex0    : TEXCOORD0;\
					float2 Tex1    : TEXCOORD1;\
					float4 Tangent : TEXCOORD2;\
					float3 Normal  : TEXCOORD3;\
					float3 tex_XYZ : TEXCOORD4;\
					float  Fog     : FOG

#define	PS_STDIN	float2 Tex0  : TEXCOORD0,float2 Tex1  : TEXCOORD1, float4 Tangent : TEXCOORD2, float3 Normal:TEXCOORD3, float3 tex_XYZ : TEXCOORD4

struct STD_OUT
{
	float4 Pos;
	float4 WorldPos;
	float2 Tex0;
	float2 Tex1;
	float3 Normal;
	float4 Tangent;
	float  Fog;
};

#define	VS_INPUT	float3 Pos  : POSITION, float3 Norm : NORMAL, float4 Tangent : TANGENT, float2 Tex0  : TEXCOORD0, float2 Tex1  : TEXCOORD1

#define	STDOUT		std_out = ComputeStdOut(Pos, Norm, Tangent, Tex0, Tex1);\
					Out.Pos = std_out.Pos;\
					Out.Normal = std_out.Normal;\
					Out.Tex0 = std_out.Tex0;\
					Out.Tex1 = std_out.Tex1;\
					Out.Tangent = std_out.Tangent;\
					Out.Fog = std_out.Fog

#define	STD_TEXLOAD	texcol = tex2D(smp0, Tex0)

STD_OUT ComputeStdOut(float3 pos, float3 norm, float4 tangent, float2 tex0, float2 tex1)
{
	STD_OUT Out;
	Out.Pos = float4(pos,1);

	float f;
	Out.WorldPos = mul(float4(pos, 1), World);
	Out.Normal = normalize(mul(norm, (float3x3)World));
	Out.Tangent = float4(normalize(mul(tangent, (float3x3)World)), tangent.w);

	float3 P = mul(Out.WorldPos, View);  // position (view space)

	f = (fog_end - P.z) * fog_idelta;
	Out.Fog = clamp(f,0,1);
	Out.Pos  = mul(float4(P, 1), Projection);             // position (projected)
	Out.Tex0  = tex0;
	Out.Tex1  = tex1;

	return Out;
}

float3 DirLight(float3 n)
{
	float cosalpha = dot(n, -dl_dir);
	if (cosalpha < 0) cosalpha = 0;
	return dl_amb + dl_dif * cosalpha;
}

float3 OmniDistance(float4 wpos, float4 oloc, float orad)
{
	return (wpos-oloc) / orad;
}

float3 CubeNormalize(float3 vec)
{
	return 2.0 * texCUBE(smp_nor, vec) - 1.0;
}

float3 OmniLight(float3 n, float4 dif, float4 dist, float range, float c_att, float l_att)
{
	float4 w;
	float q,x,y;
	float clamped;
	float3 idist;
	w = dist*dist;
	q = 1-(w.x+w.y+w.z);
	if (q >= 0) x = c_att; else x = 0;
	y = q * l_att + x;
	dif *= y;
	clamped = saturate(dot(CubeNormalize(dist.xyz),-n));
	w = saturate(dif * clamped); //SAT
	return w.xyz;
}

float3 Specular(float3 n, float3 xyz, float spec_map_val)
{
	float3 camera_dir;
	camera_dir = normalize(xyz - camera_loc);

	float3 r_vec;

	r_vec = reflect(dl_dir, n);
	float specmul = saturate(dot(-camera_dir, r_vec));
	float cosalpha = pow(specmul, glossiness) * specular * spec_map_val;
	return float4(float3(cosalpha,cosalpha,cosalpha) * dl_dif, cosalpha);
	
}

float3 ComputeNormal(float4 Tangent, float3 Normal, float4 ncol)
{
	float4 U;
	float4 V;
	float4 W;
	float3 n;

	U = Tangent;
	W = float4(Normal, 1);
	V = float4(cross(W, U), 1) * Tangent.w;

	n.x = dot ( float3(U.x,V.x,W.x), ncol );
	n.y = dot ( float3(U.y,V.y,W.y), ncol );
	n.z = dot ( float3(U.z,V.z,W.z), ncol );
	
	return n;
}

/////////////////////////////////////////
//                                     //
// MAIN SHADER PROGRAM - 0 Omni lights //
//                                     //
/////////////////////////////////////////
struct VS_OUTPUT_0L
{
	VS_STDOUT;
};

VS_OUTPUT_0L VS(VS_INPUT)
{
	VS_OUTPUT_0L Out;
	STD_OUT std_out;

	STDOUT;
	Out.tex_XYZ = std_out.WorldPos;
	
	return Out;
}

float4 PS(
    PS_STDIN
    ) : COLOR
{
	float3 n,lcol;
	float4 texcol;
	float4 suma;

	float4 ncol = tex2D(smp1_nm, Tex1) * 2 - 1;
	n = ComputeNormal(Tangent, Normal, ncol);

	lcol = DirLight(n);

	suma.xyz = lcol;
	suma.w = g_alpha;

	STD_TEXLOAD;

	float3 spec = Specular(n, tex_XYZ, texcol.a);
	texcol.a = 1;
	float4 retv = texcol * suma;
	retv.xyz += spec;

	return retv;
}

////////////////////////////////////////
//                                    //
// MAIN SHADER PROGRAM - 1 Omni light //
//                                    //
////////////////////////////////////////
struct VS_OUTPUT_1L
{
	VS_STDOUT;
	float3 L0_dist: TEXCOORD6;
};

VS_OUTPUT_1L VS_1L(VS_INPUT)
{
	VS_OUTPUT_1L Out;
	STD_OUT std_out;

	STDOUT;
	Out.tex_XYZ = std_out.WorldPos;
	
	Out.L0_dist = OmniDistance(std_out.WorldPos, omni0_loc,omni0_var.x);

	return Out;
}
float4 PS_1L(
    PS_STDIN,
    float3 L0_dist: TEXCOORD6
    ) : COLOR
{
	float3 n,lcol;
	float4 texcol;
	float4 suma;

	float4 ncol = tex2D(smp1_nm, Tex1) * 2 - 1;
	n = ComputeNormal(Tangent, Normal, ncol);

	lcol = DirLight(n);
	lcol += OmniLight(n, omni0_dif, float4(L0_dist,1), omni0_var.x, omni0_var.y, omni0_var.z);
	suma.xyz = lcol;
	suma.w = g_alpha;

	STD_TEXLOAD;
	float3 spec = Specular(n, tex_XYZ, texcol.a);
	texcol.a = 1;
	float4 retv = texcol * suma;
	retv.xyz += spec;

	return retv;
}
/////////////////////////////////////////
//                                     //
// MAIN SHADER PROGRAM - 2 Omni lights //
//                                     //
/////////////////////////////////////////
struct VS_OUTPUT_2L
{
	VS_STDOUT;
	float3 L1_dist: TEXCOORD5;
	float3 L0_dist: TEXCOORD6;
};

VS_OUTPUT_2L VS_2L(VS_INPUT)
{
	VS_OUTPUT_2L Out;
	STD_OUT std_out;

	STDOUT;
	Out.tex_XYZ = std_out.WorldPos;
	
	Out.L0_dist = OmniDistance(std_out.WorldPos, omni0_loc,omni0_var.x);
	Out.L1_dist = OmniDistance(std_out.WorldPos, omni1_loc,omni1_var.x);

	return Out;
}
float4 PS_2L(
    PS_STDIN,
    float3 L1_dist: TEXCOORD5,
    float3 L0_dist: TEXCOORD6
    ) : COLOR
{
	float3 n,lcol;
	float4 texcol;
	float4 suma;

	float4 ncol = tex2D(smp1_nm, Tex1) * 2 - 1;
	n = ComputeNormal(Tangent, Normal, ncol);

	lcol = DirLight(n);
	lcol += OmniLight(n, omni0_dif, float4(L0_dist,1), omni0_var.x, omni0_var.y, omni0_var.z);
	lcol += OmniLight(n, omni1_dif, float4(L1_dist,1), omni1_var.x, omni1_var.y, omni1_var.z);
	suma.xyz = lcol;
	suma.w = g_alpha;

	STD_TEXLOAD;
	float3 spec = Specular(n, tex_XYZ, texcol.a);
	texcol.a = 1;
	float4 retv = texcol * suma;
	retv.xyz += spec;

	return retv;
}

///////////////////////////////////////
//                                   //
// ADD SHADER PROGRAM - 1 Omni light //
//                                   //
///////////////////////////////////////
VS_OUTPUT_1L VS_A1L(VS_INPUT)
{
	VS_OUTPUT_1L Out;
	STD_OUT std_out;

	STDOUT;
	Out.tex_XYZ = std_out.WorldPos;

	Out.L0_dist = OmniDistance(std_out.WorldPos, omni2_loc,omni2_var.x);

	return Out;
}

float4 PS_A1L(
	PS_STDIN,
    float3 L2_dist: TEXCOORD6
    ) : COLOR
{
	float3 n,lcol;
	float4 texcol;
	float4 suma;

	float4 ncol = tex2D(smp1_nm, Tex1) * 2 - 1;
	n = ComputeNormal(Tangent, Normal, ncol);

	lcol = OmniLight(n, omni2_dif, float4(L2_dist,1), omni2_var.x, omni2_var.y, omni2_var.z);
	suma.xyz = lcol;
	suma.w = g_alpha;
	STD_TEXLOAD;
	return texcol * suma;
}

///////////////////////////////////////
//                                   //
// ADD SHADER PROGRAM - 2 Omni light //
//                                   //
///////////////////////////////////////
VS_OUTPUT_2L VS_A2L(VS_INPUT)
{
	VS_OUTPUT_2L Out;
	STD_OUT std_out;

	STDOUT;
	Out.tex_XYZ = std_out.WorldPos;

	Out.L0_dist = OmniDistance(std_out.WorldPos, omni2_loc,omni2_var.x);
	Out.L1_dist = OmniDistance(std_out.WorldPos, omni3_loc,omni3_var.x);

	return Out;
}

float4 PS_A2L(
	PS_STDIN,
    float3 L3_dist: TEXCOORD5,
    float3 L2_dist: TEXCOORD6
    ) : COLOR
{
	float3 n,lcol;
	float4 texcol;
	float4 suma;

	float4 ncol = tex2D(smp1_nm, Tex1) * 2 - 1;
	n = ComputeNormal(Tangent, Normal, ncol);

	lcol  = OmniLight(n, omni2_dif, float4(L2_dist,1), omni2_var.x, omni2_var.y, omni2_var.z);
	lcol += OmniLight(n, omni3_dif, float4(L3_dist,1), omni3_var.x, omni3_var.y, omni3_var.z);
	suma.xyz = lcol;
	suma.w = g_alpha;
	STD_TEXLOAD;
	return texcol * suma;
}

///////////////////////////////////////
//                                   //
// ADD SHADER PROGRAM - 3 Omni light //
//                                   //
///////////////////////////////////////
struct VS_OUTPUT_3L
{
	VS_STDOUT;
	float3 L2_dist: TEXCOORD5;
	float3 L1_dist: TEXCOORD6;
	float3 L0_dist: TEXCOORD7;
};
VS_OUTPUT_3L VS_A3L(VS_INPUT)
{
	VS_OUTPUT_3L Out;
	STD_OUT std_out;

	STDOUT;
	Out.tex_XYZ = std_out.WorldPos;

	Out.L0_dist = OmniDistance(std_out.WorldPos, omni2_loc,omni2_var.x);
	Out.L1_dist = OmniDistance(std_out.WorldPos, omni3_loc,omni3_var.x);
	Out.L2_dist = OmniDistance(std_out.WorldPos, omni4_loc,omni4_var.x);

	return Out;
}

float4 PS_A3L(
	PS_STDIN,
    float3 L4_dist: TEXCOORD5,
    float3 L3_dist: TEXCOORD6,
    float3 L2_dist: TEXCOORD7
    ) : COLOR
{
	float3 n,lcol;
	float4 texcol;
	float4 suma;

	float4 ncol = tex2D(smp1_nm, Tex1) * 2 - 1;
	n = ComputeNormal(Tangent, Normal, ncol);

	lcol  = OmniLight(n, omni2_dif, float4(L2_dist,1), omni2_var.x, omni2_var.y, omni2_var.z);
	lcol += OmniLight(n, omni3_dif, float4(L3_dist,1), omni3_var.x, omni3_var.y, omni3_var.z);
	lcol += OmniLight(n, omni4_dif, float4(L4_dist,1), omni4_var.x, omni4_var.y, omni4_var.z);
	suma.xyz = lcol;
	suma.w = g_alpha;
	STD_TEXLOAD;
	return texcol * suma;
}







///////////////////////////////////////
//                                   //
// ADD SHADER PROGRAM - 1 Omni light //
//                                   //
///////////////////////////////////////
VS_OUTPUT_1L VS_B1L(VS_INPUT)
{
	VS_OUTPUT_1L Out;
	STD_OUT std_out;

	STDOUT;
	Out.tex_XYZ = std_out.WorldPos;

	Out.L0_dist = OmniDistance(std_out.WorldPos, omni5_loc,omni5_var.x);

	return Out;
}

float4 PS_B1L(
	PS_STDIN,
    float3 L5_dist: TEXCOORD6
    ) : COLOR
{
	float3 n,lcol;
	float4 texcol;
	float4 suma;

	float4 ncol = tex2D(smp1_nm, Tex1) * 2 - 1;
	n = ComputeNormal(Tangent, Normal, ncol);

	lcol = OmniLight(n, omni5_dif, float4(L5_dist,1), omni5_var.x, omni5_var.y, omni5_var.z);
	suma.xyz = lcol;
	suma.w = g_alpha;
	STD_TEXLOAD;
	return texcol * suma;
}

///////////////////////////////////////
//                                   //
// ADD SHADER PROGRAM - 2 Omni light //
//                                   //
///////////////////////////////////////
VS_OUTPUT_2L VS_B2L(VS_INPUT)
{
	VS_OUTPUT_2L Out;
	STD_OUT std_out;

	STDOUT;
	Out.tex_XYZ = std_out.WorldPos;

	Out.L0_dist = OmniDistance(std_out.WorldPos, omni5_loc,omni5_var.x);
	Out.L1_dist = OmniDistance(std_out.WorldPos, omni6_loc,omni6_var.x);

	return Out;
}

float4 PS_B2L(
	PS_STDIN,
    float3 L6_dist: TEXCOORD5,
    float3 L5_dist: TEXCOORD6
    ) : COLOR
{
	float3 n,lcol;
	float4 texcol;
	float4 suma;

	float4 ncol = tex2D(smp1_nm, Tex1) * 2 - 1;
	n = ComputeNormal(Tangent, Normal, ncol);

	lcol  = OmniLight(n, omni5_dif, float4(L5_dist,1), omni5_var.x, omni5_var.y, omni5_var.z);
	lcol += OmniLight(n, omni6_dif, float4(L6_dist,1), omni6_var.x, omni6_var.y, omni6_var.z);
	suma.xyz = lcol;
	suma.w = g_alpha;
	STD_TEXLOAD;
	return texcol * suma;
}

technique tech_0L
{
	pass P0 {
//		Sampler[0] = (smp0);
//		Sampler[1] = (smp1_nm);
		VertexShader = compile vs_1_1 VS();
		PixelShader = compile ps_2_0 PS();
	}
}

technique tech_1L
{
	pass P0 {
		VertexShader = compile vs_1_1 VS_1L();
		PixelShader = compile ps_2_0 PS_1L();
	}
}
technique tech_2L
{
	pass P0 {
		VertexShader = compile vs_1_1 VS_2L();
		PixelShader = compile ps_2_0 PS_2L();
	}
}

technique tech_3L
{
	pass P0 {
		VertexShader = compile vs_1_1 VS_2L();
		PixelShader = compile ps_2_0 PS_2L();
	}
	pass P0 {
		AlphaBlendEnable = TRUE;
		SrcBlend = ONE;
		DestBlend = ONE;
		VertexShader = compile vs_1_1 VS_A1L();
		PixelShader = compile ps_2_0 PS_A1L();
	}
}

technique tech_4L
{
	pass P0 {
		VertexShader = compile vs_1_1 VS_2L();
		PixelShader = compile ps_2_0 PS_2L();
	}
	pass P1 {
		AlphaBlendEnable = TRUE;
		SrcBlend = ONE;
		DestBlend = ONE;
		VertexShader = compile vs_1_1 VS_A2L();
		PixelShader = compile ps_2_0 PS_A2L();
	}
}

technique tech_5L
{
	pass P0 {
		VertexShader = compile vs_1_1 VS_2L();
		PixelShader = compile ps_2_0 PS_2L();
	}
	pass P1 {
		AlphaBlendEnable = TRUE;
		SrcBlend = ONE;
		DestBlend = ONE;
		VertexShader = compile vs_1_1 VS_A3L();
		PixelShader = compile ps_2_0 PS_A3L();
	}
}

technique tech_6L
{
	pass P0 {
		VertexShader = compile vs_1_1 VS_2L();
		PixelShader = compile ps_2_0 PS_2L();
	}
	pass P1 {
		AlphaBlendEnable = TRUE;
		SrcBlend = ONE;
		DestBlend = ONE;
		VertexShader = compile vs_1_1 VS_A3L();
		PixelShader = compile ps_2_0 PS_A3L();
	}
	pass P2 {
		AlphaBlendEnable = TRUE;
		SrcBlend = ONE;
		DestBlend = ONE;
		VertexShader = compile vs_1_1 VS_B1L();
		PixelShader = compile ps_2_0 PS_B1L();
	}
}

technique tech_7L
{
	pass P0 {
		VertexShader = compile vs_1_1 VS_2L();
		PixelShader = compile ps_2_0 PS_2L();
	}
	pass P1 {
		AlphaBlendEnable = TRUE;
		SrcBlend = ONE;
		DestBlend = ONE;
		VertexShader = compile vs_1_1 VS_A3L();
		PixelShader = compile ps_2_0 PS_A3L();
	}
	pass P2 {
		AlphaBlendEnable = TRUE;
		SrcBlend = ONE;
		DestBlend = ONE;
		VertexShader = compile vs_1_1 VS_B2L();
		PixelShader = compile ps_2_0 PS_B2L();
	}
}
