// transformations
float4x4 World      : WORLD;
float4x4 View       : VIEW;
float4x4 Projection : PROJECTION;
float3   dl_dir;
float3   dl_dif;
float3   dl_amb;
float	 g_alpha;
float	fog_idelta;
float	fog_end;
float4	omni0_dif,omni0_loc,omni0_var;
float4	omni1_dif,omni1_loc,omni1_var;
float4	omni2_dif,omni2_loc,omni2_var;
float4	omni3_dif,omni3_loc,omni3_var;
float4	omni4_dif,omni4_loc,omni4_var;
float4	omni5_dif,omni5_loc,omni5_var;
float4	omni6_dif,omni6_loc,omni6_var;

sampler smp0 : register(s0);
sampler smp1 : register(s1);

#define	VS_STDOUT	float4 Pos    : POSITION;\
					float2 Tex0   : TEXCOORD0;\
					float2 Tex1   : TEXCOORD1;\
					float4 Tangent: TEXCOORD6;\
					float  Fog    : FOG;\
					float3 Normal : TEXCOORD7

#define	PS_STDIN	float2 Tex0  : TEXCOORD0,float2 Tex1  : TEXCOORD1,float4 Tangent : TEXCOORD6, float3 Normal:TEXCOORD7

struct STD_OUT
{
	float4 Pos;
	float4 WorldPos;
	float2 Tex0;
	float2 Tex1;
	float3 Normal;
	float4 Tangent;
	float  Fog;
};

#define	VS_INPUT	float3 Pos  : POSITION, float3 Norm : NORMAL, float4 Tangent : TANGENT, float2 Tex0  : TEXCOORD0, float2 Tex1  : TEXCOORD1

#define	STDOUT		std_out = ComputeStdOut(Pos, Norm, Tangent, Tex0, Tex1);\
					Out.Pos = std_out.Pos;\
					Out.Normal = std_out.Normal;\
					Out.Tex0 = std_out.Tex0;\
					Out.Tex1 = std_out.Tex1;\
					Out.Tangent = std_out.Tangent;\
					Out.Fog = std_out.Fog

#define	STD_TEXLOAD	texcol = tex2D(smp0, Tex0)

STD_OUT ComputeStdOut(float3 pos, float3 norm, float4 tangent, float2 tex0, float2 tex1)
{
	STD_OUT Out;
	Out.Pos = float4(pos,1);

	float f;
	Out.WorldPos = mul(float4(pos, 1), World);
	Out.Normal = mul(norm, (float3x3)World);
	Out.Tangent = float4(mul(tangent, (float3x3)World), tangent.w);
	
	float3 P = mul(Out.WorldPos, View);  // position (view space)

	f = (fog_end - P.z) * fog_idelta;
	Out.Fog = clamp(f,0,1);
	Out.Pos  = mul(float4(P, 1), Projection);             // position (projected)
	Out.Tex0  = tex0;
	Out.Tex1  = tex1;

	return Out;
}

float3 DirLight(float3 n)
{
	float cosalpha = dot(n, -dl_dir);
	if (cosalpha < 0) cosalpha = 0;
	return dl_amb + dl_dif * cosalpha;
}

float3 OmniDistance(float4 wpos, float4 oloc, float orad)
{
	return (wpos-oloc) / orad;
}

float3 OmniLight(float3 n, float4 dif, float4 dist, float range, float c_att, float l_att)
{
	float4 w;
	float q,x,y;
	float clamped;
	float3 idist;
	w = dist*dist;
	q = 1-(w.x+w.y+w.z);
	if (q >= 0) x = c_att; else x = 0;
	y = q * l_att + x;
	dif *= y;
	clamped = saturate(dot(normalize(dist.xyz),-n));
	w = saturate(dif * clamped); //SAT
	return w.xyz;
}

/////////////////////////////////////////
//                                     //
// MAIN SHADER PROGRAM - 0 Omni lights //
//                                     //
/////////////////////////////////////////
struct VS_OUTPUT_0L
{
	VS_STDOUT;
};

VS_OUTPUT_0L VS(VS_INPUT)
{
	VS_OUTPUT_0L Out;
	STD_OUT std_out;

	STDOUT;
	
	return Out;
}

float4 PS(
    PS_STDIN
    ) : COLOR
{
	float3 n,lcol;
	float4 texcol;
	float4 suma;
	float4 ncol;

	STD_TEXLOAD;
	ncol = tex2D(smp1, Tex1) * 2 - 1;

	float4 U;
	float4 V;
	float4 W;

	U = Tangent;
	W = float4(Normal, 1);
	V = float4(cross(W, U), 1) * Tangent.w;

/*	float3 ldir;
	ldir.x = dot ( U.xyz, -dl_dir );
	ldir.y = dot ( V.xyz, -dl_dir );
	ldir.z = dot ( W.xyz, -dl_dir );

	float cosalpha = dot(ncol.xyz, ldir);
	if (cosalpha < 0) cosalpha = 0;
	lcol = dl_amb + dl_dif * cosalpha;*/

	n.x = dot ( float3(U.x,V.x,W.x), ncol );
	n.y = dot ( float3(U.y,V.y,W.y), ncol );
	n.z = dot ( float3(U.z,V.z,W.z), ncol );
	float cosalpha = dot(n, -dl_dir);
	if (cosalpha < 0) cosalpha = 0;
	lcol = DirLight(n);

	suma.xyz = lcol;
	suma.w = g_alpha;
	return texcol * suma;

}

/////////////////////////////////////////
//                                     //
// MAIN SHADER PROGRAM - 0 Omni lights //
//                                     //
/////////////////////////////////////////
struct VS_OUTPUT_1L
{
	VS_STDOUT;
	float3 L0_dist: TEXCOORD5;
};

VS_OUTPUT_1L VS1L(VS_INPUT)
{
	VS_OUTPUT_1L Out;
	STD_OUT std_out;

	STDOUT;
	Out.L0_dist = OmniDistance(std_out.WorldPos, omni0_loc,omni0_var.x);
	return Out;
}

float4 PS1L(
    PS_STDIN,
    float3 L0_dist: TEXCOORD5
    ) : COLOR
{
	float3 n,lcol;
	float4 texcol;
	float4 suma;
	float4 ncol;

	STD_TEXLOAD;
	ncol = tex2D(smp1, Tex1) * 2 - 1;

	float4 U;
	float4 V;
	float4 W;

	U = Tangent;
	W = float4(Normal, 1);
	V = float4(cross(U, W), 1) * Tangent.w;

	n.x = dot ( float3(U.x,V.x,W.x), ncol );
	n.y = dot ( float3(U.y,V.y,W.y), ncol );
	n.z = dot ( float3(U.z,V.z,W.z), ncol );
	lcol = DirLight(n);

	lcol += OmniLight(n, omni0_dif, float4(L0_dist,1), omni0_var.x, omni0_var.y, omni0_var.z);
	suma.xyz = lcol;
	suma.w = g_alpha;

	STD_TEXLOAD;
	return texcol * suma;
}

technique tech_0L
{
	pass P0 {
		VertexShader = compile vs_1_1 VS();
		PixelShader = compile ps_2_0 PS();
	}
}

technique tech_1L
{
	pass P0 {
		VertexShader = compile vs_1_1 VS1L();
		PixelShader = compile ps_2_0 PS1L();
	}
}
